@@ -29,6 +29,7 @@
 #include "ephy-file-helpers.h"
 #include "ephy-dnd.h"
 #include "ephy-embed.h"
+#include "ephy-embed-utils.h"
 #include "ephy-window.h"
 #include "ephy-shell.h"
 #include "ephy-spinner.h"
@@ -36,15 +37,7 @@
 #include "ephy-debug.h"
 
 #include <glib/gi18n.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkaccelgroup.h>
-#include <gtk/gtkiconfactory.h>
+#include <gtk/gtk.h>
 
 #define TAB_WIDTH_N_CHARS 15
 
@@ -53,6 +46,8 @@
 
 #define INSANE_NUMBER_OF_URLS 20
 
+#define EPHY_NOTEBOOK_TAB_GROUP_ID (GUINT_TO_POINTER (42))
+
 #define EPHY_NOTEBOOK_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NOTEBOOK, EphyNotebookPrivate))
 
 struct _EphyNotebookPrivate
@@ -77,8 +72,9 @@ static void ephy_notebook_remove	 (GtkContainer *container,
 
 static const GtkTargetEntry url_drag_types [] = 
 {
+        { "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, 0 },
 	{ EPHY_DND_URI_LIST_TYPE,   0, 0 },
-	{ EPHY_DND_URL_TYPE,	    0, 1 }
+	{ EPHY_DND_URL_TYPE,	    0, 1 },
 };
 
 enum
@@ -357,13 +353,21 @@ ephy_notebook_switch_page_cb (GtkNotebook *notebook,
 }
 
 static void
-notebook_drag_data_received_cb (GtkWidget* widget, GdkDragContext *context,
-				gint x, gint y, GtkSelectionData *selection_data,
-				guint info, guint time, EphyEmbed *embed)
+notebook_drag_data_received_cb (GtkWidget* widget,
+                                GdkDragContext *context,
+				int x,
+                                int y,
+                                GtkSelectionData *selection_data,
+				guint info,
+                                guint time,
+                                EphyEmbed *embed)
 {
 	EphyWindow *window;
 	GtkWidget *notebook;
 
+        if (selection_data->target == gdk_atom_intern_static_string ("GTK_NOTEBOOK_TAB"))
+                return;
+
 	g_signal_stop_emission_by_name (widget, "drag_data_received");
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL)) return;
@@ -450,12 +454,15 @@ static void
 ephy_notebook_init (EphyNotebook *notebook)
 {
 	EphyNotebookPrivate *priv;
+        GtkWidget *widget = GTK_WIDGET (notebook);
+        GtkNotebook *gnotebook = GTK_NOTEBOOK (notebook);
 
 	priv = notebook->priv = EPHY_NOTEBOOK_GET_PRIVATE (notebook);
 
-	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
-	gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
+	gtk_notebook_set_scrollable (gnotebook, TRUE);
+	gtk_notebook_set_show_border (gnotebook, FALSE);
+	gtk_notebook_set_show_tabs (gnotebook, FALSE);
+        gtk_notebook_set_group (gnotebook, EPHY_NOTEBOOK_TAB_GROUP_ID);
 
 	priv->show_tabs = TRUE;
 	priv->dnd_enabled = TRUE;
@@ -470,12 +477,10 @@ ephy_notebook_init (EphyNotebook *notebook)
 	g_signal_connect (notebook, "drag-data-received",
 			  G_CALLBACK (notebook_drag_data_received_cb),
 			  NULL);
-	gtk_drag_dest_set (GTK_WIDGET (notebook),
-			   GTK_DEST_DEFAULT_MOTION |
-			   GTK_DEST_DEFAULT_DROP,
+	gtk_drag_dest_set (widget, 0,
 			   url_drag_types, G_N_ELEMENTS (url_drag_types),
 			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
-	gtk_drag_dest_add_text_targets (GTK_WIDGET(notebook));
+	gtk_drag_dest_add_text_targets (widget);
 
 	priv->tabs_vis_notifier_id = eel_gconf_notification_add
 		(CONF_ALWAYS_SHOW_TABS_BAR,
@@ -496,7 +501,7 @@ ephy_notebook_finalize (GObject *object)
 }
 
 static void
-sync_load_status (EphyEmbed *embed, GParamSpec *pspec, GtkWidget *proxy)
+sync_load_status (EphyWebView *view, GParamSpec *pspec, GtkWidget *proxy)
 {
 	GtkWidget *spinner, *icon;
 
@@ -504,7 +509,7 @@ sync_load_status (EphyEmbed *embed, GParamSpec *pspec, GtkWidget *proxy)
 	icon = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "icon"));
 	g_return_if_fail (spinner != NULL && icon != NULL);
 
-	if (ephy_embed_get_load_status (embed))
+	if (ephy_web_view_get_load_status (view))
 	{
 		gtk_widget_hide (icon);
 		gtk_widget_show (spinner);
@@ -519,19 +524,19 @@ sync_load_status (EphyEmbed *embed, GParamSpec *pspec, GtkWidget *proxy)
 }
 
 static void
-sync_icon (EphyEmbed *embed,
+sync_icon (EphyWebView *view,
 	   GParamSpec *pspec,
 	   GtkImage *icon)
 {
-	gtk_image_set_from_pixbuf (icon, ephy_embed_get_icon (embed));
+	gtk_image_set_from_pixbuf (icon, ephy_web_view_get_icon (view));
 }
 
 static void
-sync_label (EphyEmbed *embed, GParamSpec *pspec, GtkWidget *label)
+sync_label (EphyWebView *view, GParamSpec *pspec, GtkWidget *label)
 {
 	const char *title;
 
-	title = ephy_embed_get_title (embed);
+	title = ephy_web_view_get_title (view);
 
 	gtk_label_set_text (GTK_LABEL (label), title);
 
@@ -582,6 +587,7 @@ static GtkWidget *
 build_tab_label (EphyNotebook *nb, EphyEmbed *embed)
 {
 	GtkWidget *hbox, *label, *close_button, *image, *spinner, *icon;
+	EphyWebView *view;
 
 	/* set hbox spacing and label padding (see below) so that there's an
 	 * equal amount of space around the label */
@@ -645,15 +651,16 @@ build_tab_label (EphyNotebook *nb, EphyEmbed *embed)
 	g_object_set_data (G_OBJECT (hbox), "close-button", close_button);
 
 	/* Hook the label up to the tab properties */
-	sync_icon (embed, NULL, GTK_IMAGE (icon));
-	sync_label (embed, NULL, label);
-	sync_load_status (embed, NULL, hbox);
+	view = EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed);
+	sync_icon (view, NULL, GTK_IMAGE (icon));
+	sync_label (view, NULL, label);
+	sync_load_status (view, NULL, hbox);
 
-	g_signal_connect_object (embed, "notify::icon",
+	g_signal_connect_object (view, "notify::icon",
 				 G_CALLBACK (sync_icon), icon, 0);
-	g_signal_connect_object (embed, "notify::title",
+	g_signal_connect_object (view, "notify::embed-title",
 				 G_CALLBACK (sync_label), label, 0);
-	g_signal_connect_object (embed, "notify::load-status",
+	g_signal_connect_object (view, "notify::load-status",
 				 G_CALLBACK (sync_load_status), hbox, 0);
 
 	return hbox;
@@ -707,6 +714,7 @@ ephy_notebook_insert_page (GtkNotebook *gnotebook,
 										 position);
 
 	gtk_notebook_set_tab_reorderable (gnotebook, tab_widget, TRUE);
+        gtk_notebook_set_tab_detachable (gnotebook, tab_widget, TRUE);
 
 	return position;
 }
@@ -781,6 +789,7 @@ ephy_notebook_remove (GtkContainer *container,
 	EphyNotebookPrivate *priv = notebook->priv;
 	GtkWidget *tab_label, *tab_label_label, *tab_label_icon;
 	int position, curr;
+	EphyWebView *view;
 
 	g_assert (EPHY_IS_EMBED (tab_widget));
 
@@ -800,12 +809,14 @@ ephy_notebook_remove (GtkContainer *container,
 	tab_label_icon = g_object_get_data (G_OBJECT (tab_label), "icon");
 	tab_label_label = g_object_get_data (G_OBJECT (tab_label), "label");
 
+	view = EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (tab_widget);
+
 	g_signal_handlers_disconnect_by_func
-		(tab_widget, G_CALLBACK (sync_icon), tab_label_icon);
+		(view, G_CALLBACK (sync_icon), tab_label_icon);
 	g_signal_handlers_disconnect_by_func
-		(tab_widget, G_CALLBACK (sync_label), tab_label_label);
+		(view, G_CALLBACK (sync_label), tab_label_label);
 	g_signal_handlers_disconnect_by_func
-	  (tab_widget, G_CALLBACK (sync_load_status), tab_label);
+	  (view, G_CALLBACK (sync_load_status), tab_label);
 
 	GTK_CONTAINER_CLASS (ephy_notebook_parent_class)->remove (container, tab_widget);
 
